﻿--Bar creation module

local bars = {}
local tsort = table.sort

function SN:AnnounceCD(cooldown, caster)
	
	--Is the announce setting and in a raid and are you the leader...otherwise no spam for you
	if (SN.db.profile.CD.castannounce) and (SN.db.profile.CD.raid) and (UnitIsGroupLeader("player")) then
		local _, duration = GetSpellCooldown(cooldown.id)
		local dur = cooldown.CD / 60
		local class, classFileName, classID = UnitClass(caster)
		
		if GetNumGroupMembers() > 1 and IsInRaid() then
			SendChatMessage(caster.." casts "..GetSpellLink(cooldown.id).." "..dur.."min".." CD", "RAID")
		elseif GetNumGroupMembers() > 1 and not IsInRaid() then
			SendChatMessage(caster.." casts "..GetSpellLink(cooldown.id).." "..dur.."min".." CD", "PARTY")
		end
	end
end


function SN:CreateBar(frame, cooldown, duration, caster, guid, spell)

	--Make our lives easier with some variables
	local v = SN.Media
	local class, classFileName, classID = UnitClass(caster)
	local bar

	--Start the CD Timer
	bar = SN.timers:New(v.media.minTex, frame:GetWidth(), frame.height)
	bar:SetParent(frame)
	--Check to see if ElvUI is on so we can make the frames match
	if (IsAddOnLoaded("ElvUI")) then
		SN:SNCreateBG(bar)
	else
	--Standard Backdrop settings
		bar:SetAlpha(1)
		bar:SetBackdrop({
			bgFile = v.media.blankTex, 
			edgeFile = v.media.blankTex, 
			tile = false, tileSize = 0, edgeSize = 2, 
			insets = { left = -2, right = 2, top = 2, bottom = -2 }
		})
		bar:SetBackdropColor(0, 0, 0, 0)
		bar:SetBackdropBorderColor(0.1, 0.1, 0.1, 1)
	end
	
	--Grab the player's class color and the icon of the spell
	local colortmp = RAID_CLASS_COLORS[classFileName]
	color = { colortmp.r, colortmp.g, colortmp.b }
	local _, _, icon = GetSpellInfo(cooldown.id)
	bar:SetIcon(icon)
	
	bar:SetColor(unpack(color))
	bar:SetDuration(duration)
	bar.candyBarDuration:SetFont(v.media.font, 16)
	bar.candyBarLabel:SetFont(v.media.font, 16)
	bar.candyBarLabel:SetJustifyH("LEFT")

	--Setup the data keys that the bar will hold
	bar:Set("SN:arg1", frame)
	bar:Set("SN:arg2", cooldown)
	bar:Set("SN:arg3", guid)
	bar:Set("SN:arg4", caster)
	bar:Set("SN:arg5", spell)
	bar:Set("SN:arg6", classID)
	bar.bartype = cooldown
	bar.TimeSinceLastUpdate = 0
	
	--Make sure only the CD monitor gets the caster and spell names
	if frame == SN.F.debuffs then
		bar:SetLabel(spell)
	elseif frame == SN.F.cooldowns then
		bar:SetLabel(caster.."       "..spell)
	end
	
	--Add some things the bar should do on the events
	bar:SetScript("OnEvent", function(self, event, ...)
		if event == "COMBAT_LOG_EVENT_UNFILTERED" then
			local _, type, _, _, sourceName, _, _, destGUID, destname, _, _ = ...
			local spellId, spellName, spellSchool = select(12, ...)

			--Kill the bar if the debuff target is dead
			if type == "SPELL_AURA_REMOVED" then
				if destGUID == self:Get("SN:arg3") and spellName == self:Get("SN:arg5") then
					self:Stop()
				end
			end

			--See if the hunter Spell Readiness was cast and then kill the CD's reset bars
			--TODO: Make this more class nuetral and allow for other classes to check this
			if type == "SPELL_CAST_SUCCESS" then
				if select(2, UnitClass("player")) == "HUNTER" then
					if spellId == 23989 then
						if self:Get("SN:arg2").reset then
							self:Stop()
						end
					end
				end
			end
		end	
	end)
	--Add bar to the appropriate group
	tinsert(frame.bars, bar)
	bar.frame = frame

	SN:Sort(frame)
	LibStub("LibCandyBar-3.0").RegisterCallback(bar, "LibCandyBar_Stop", barRemove)
	bar:Start()

	--So we can check on our combat changes
	bar:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
end

function barRemove(callback, self)

	local tmpframe = self.frame
	for i, bar in pairs(tmpframe.bars) do
		if bar.bartype.id == self.bartype.id then
			tmpframe.bars[i] = nil
		end
	end
	SN:Sort(tmpframe)
end

function SN:SortBars(a, b, field)
	-- sort using the time remaining
	if field == "time" then
		return (a.remaining < b.remaining)
	-- sort using the bar's label
	elseif field == "label" then
		return (a.candyBarLabel < b.candyBarLabel)
	-- sort using a data field
	else
		return (a:Get(field) < b:Get(field))
	end
end

function SN:Sort(frame)
	if frame.bars == nil then return end
	local sh = frame.spacing_h or 0
	local sv = frame.spacing_v or 0

	local tmp = {}
	for i, bar in pairs(frame.bars) do
		tmp[#tmp + 1] = bar
	end
	tsort(tmp, function(a, b) return SN:SortBars(a, b, frame.sortfield) end)
	local lastBar = false
	local rowFirst = false
	local barNo = 0

	local barMax
	if frame.maxbars > 0 and frame.maxbars <= #tmp then
		barMax = frame.maxbars
	else
		barMax = #tmp
	end

	local startpoint = frame.startpoint
	local width = frame:GetWidth()

	local x, y, z, bar
	if frame.sortasc then
		x = barMax
		y = 1
		z = -1
	else
		x = 1
		y = barMax
		z = 1
	end

	for i = x, y, z do
		bar = tmp[i]
		bar:Show()
		bar:ClearAllPoints()
		bar:SetWidth(width)
  		if rowFirst then
			sv = frame.spacing_v
		else
			sv = 0
		end

		if startpoint == "topleft" then
			bar:SetPoint("BOTTOMLEFT", rowFirst or frame, "TOPLEFT", 0, sv)
			rowFirst = bar
		elseif startpoint == "topright" then
			bar:SetPoint("BOTTOMRIGHT", rowFirst or frame, "TOPRIGHT", 0, sv)
			rowFirst = bar
		elseif startpoint == "bottomright" then
			bar:SetPoint("TOPRIGHT", rowFirst or frame, "BOTTOMRIGHT", 0, -sv)
			rowFirst = bar
		else
			bar:SetPoint("TOPLEFT", rowFirst or frame, "BOTTOMLEFT", 0, -sv)
			rowFirst = bar
		end
		lastBar = bar
		barNo = barNo + 1
	end

	if frame.maxbars > 0 and frame.maxbars < #tmp then
		for i = frame.maxbars + 1, #tmp do
			tmp[i]:Hide()
		end
	end
end

function SN:Find(frame, key, value)
	for i, k in pairs(frame.bars) do
		if k.data[key] and k.data[key] == value then return k end
	end
	return nil
end